@@ -7,13 +7,13 @@ from importlib import import_module
 from slai.modules.parameters import from_config
 from slai.config import get_api_base_urls
 from slai.modules.runtime import detect_credentials
-from slai.clients.model import get_model_client
+from slai.clients.model import ModelClient, get_model_client
 from slai.types import ModelTypes, InvalidPayloadException, InvalidTypeException
 
 REQUESTS_TIMEOUT = 180
 
 
-def get_inference_client(*, org_name, model_name, model_version_name):
+def get_inference_client(*, model_uri):
     import_path = from_config(
         "MODEL_INFERENCE_CLIENT",
         "slai.clients.inference.ModelInferenceClient",
@@ -22,19 +22,18 @@ def get_inference_client(*, org_name, model_name, model_version_name):
     path = ".".join(import_path.split(".")[:-1])
 
     return getattr(import_module(path), class_)(
-        org_name=org_name,
-        model_name=model_name,
-        model_version_name=model_version_name,
+        model_uri=model_uri,
     )
 
 
 @lru_cache(maxsize=64)
 def _get_model_info(
-    base_url, client_id, client_secret, model_id, model_version_id=None
+    base_url,
+    client_id,
+    client_secret,
+    sandbox_id,
 ):
-    body = {"model_id": model_id}
-    if model_version_id is not None:
-        body["model_version_id"] = model_version_id
+    body = {"sandbox_id": sandbox_id}
 
     res = requests.post(
         f"{base_url}/model/info",
@@ -43,73 +42,58 @@ def _get_model_info(
         timeout=REQUESTS_TIMEOUT,
     )
     res.raise_for_status()
-    return res.json()
+    model_info = res.json()
+
+    input_schema = ModelTypes.load_schema(model_info["input_schema"])
+    output_schema = ModelTypes.load_schema(model_info["output_schema"])
+    inference_url = model_info["inference_url"]
+
+    return input_schema, output_schema, inference_url
 
 
 class ModelInferenceClient:
     BACKEND_BASE_URL, _ = get_api_base_urls()
 
-    def __init__(self, *, org_name, model_name, model_version_name=None):
+    def __init__(self, *, model_uri):
         credentials = detect_credentials()
 
         self.client_id = credentials["client_id"]
         self.client_secret = credentials["client_secret"]
-
-        self.org_name = org_name
-        self.model_name = model_name
-        self.model_version_name = model_version_name
+        self.model_uri = model_uri
 
         self._load_model()
 
     def _load_model(self):
-        self.model_client = get_model_client(
-            org_name=self.org_name,
-            model_name=self.model_name,
+        self.model_client: ModelClient = get_model_client(
+            model_uri=self.model_uri,
         )
-        self.model = self.model_client.get_model()
-
-        if self.model_version_name is not None:
-            self.model_version = self.model_client.get_model_version_by_name(
-                model_version_name=self.model_version_name
-            )
-            self.model_version_id = self.model_version["id"]
-        else:
-            self.model_version_id = None
+        self.sandbox = self.model_client.get_sandbox()
 
     def call(self, payload):
-        model_info = _get_model_info(
+        input_schema, output_schema, inference_url = _get_model_info(
             self.BACKEND_BASE_URL,
             self.client_id,
             self.client_secret,
-            self.model["id"],
-            self.model_version_id,
+            self.sandbox["short_id"],
         )
-        input_schema = ModelTypes.load_schema(model_info["input_schema"])
-        output_schema = ModelTypes.load_schema(model_info["output_schema"])
 
         try:
-            body = {
-                "model_id": self.model["id"],
-                "model_version_id": self.model_version_id,
-                "payload": ModelTypes.serialize(payload, input_schema),
-            }
+            serialized_payload = ModelTypes.serialize(payload, input_schema)
         except (InvalidTypeException, InvalidPayloadException) as e:
             print(e, e.errors)
             return None
 
-        if body.get("model_version_id") is None:
-            del body["model_version_id"]
-
         res = requests.post(
-            f"{self.BACKEND_BASE_URL}/model/call",
+            f"{inference_url}",
             auth=HTTPBasicAuth(self.client_id, self.client_secret),
-            json=body,
+            json=serialized_payload,
             timeout=REQUESTS_TIMEOUT,
         )
         res.raise_for_status()
         response_data = res.json()
 
-        if errors := response_data["result"]["result"].get("errors", []):
+        errors = response_data["result"].get("errors", [])
+        if errors:
             print("An unhandled exception occurred in your handler:\n ")
 
             for e in errors:
@@ -118,9 +102,7 @@ class ModelInferenceClient:
             return None
 
         try:
-            result = ModelTypes.deserialize(
-                response_data["result"]["result"], output_schema
-            )
+            result = ModelTypes.deserialize(response_data["result"], output_schema)
         except (InvalidTypeException, InvalidPayloadException) as e:
             print(e, e.errors)
             return None
@@ -133,7 +115,6 @@ class ModelInferenceClient:
                 self.BACKEND_BASE_URL,
                 self.client_id,
                 self.client_secret,
-                self.model["id"],
-                self.model_version_id,
+                self.sandbox["short_id"],
             )
         )
