@@ -1,10 +1,12 @@
-from abc import ABC, abstractmethod
-from importlib import import_module
 import io
 import base64
 import json
 import builtins
 import copy
+import os
+
+from abc import ABC, abstractmethod
+from importlib import import_module
 
 
 class InvalidTypeException(Exception):
@@ -23,8 +25,13 @@ class InvalidPayloadException(Exception):
 
 class ModelTypeSerializer(ABC):
     def __init__(self, *args, **kwargs):
+        self._set_attributes(**kwargs)
         return self.define(*args, **kwargs)
 
+    def _set_attributes(self, **kwargs):
+        for k, v in kwargs.items():
+            setattr(self, k, v)
+
     @abstractmethod
     def define(self, *args, **kwargs):
         pass
@@ -43,7 +50,7 @@ class ModelTypeSerializer(ABC):
 
 
 class FloatType(ModelTypeSerializer):
-    def define(self):
+    def define(self, **_):
         pass
 
     def schema(self):
@@ -67,7 +74,7 @@ class FloatType(ModelTypeSerializer):
 
 
 class StringType(ModelTypeSerializer):
-    def define(self, max_length=None):
+    def define(self, max_length=None, **_):
         self.max_length = max_length
 
     def schema(self):
@@ -93,7 +100,7 @@ class StringType(ModelTypeSerializer):
 
 
 class JsonType(ModelTypeSerializer):
-    def define(self):
+    def define(self, **_):
         pass
 
     def schema(self):
@@ -125,7 +132,7 @@ class JsonType(ModelTypeSerializer):
 
 
 class ImageType(ModelTypeSerializer):
-    def define(self, size=None, mode=None, raw=False):
+    def define(self, size=None, mode=None, raw=False, **_):
         self.size = size
         self.mode = mode
         self.raw = raw
@@ -139,6 +146,24 @@ class ImageType(ModelTypeSerializer):
         }
 
     def serialize(self, obj):
+        try:
+            PIL_Image = import_module("PIL.Image")
+        except ModuleNotFoundError:
+            raise RuntimeError("pillow_required_for_image_inputs")
+
+        # Load image from disk
+        if isinstance(obj, str) and os.path.isfile(obj):
+            obj = PIL_Image.open(obj)
+
+        # Load image from base64
+        elif isinstance(obj, str):
+            base64_data = obj
+
+            if "," in base64_data:
+                base64_data = obj.split(",")[1]
+
+            obj = PIL_Image.open(io.BytesIO(base64.b64decode(base64_data)))
+
         if self.size is not None and obj.size != self.size:
             raise InvalidTypeException("invalid_image_size")
 
@@ -153,6 +178,7 @@ class ImageType(ModelTypeSerializer):
     def deserialize(self, data):
         if self.raw:
             return str(data)
+
         try:
             PIL_Image = import_module("PIL.Image")
         except ModuleNotFoundError:
@@ -173,13 +199,18 @@ class ImageType(ModelTypeSerializer):
 
 
 class BinaryType(ModelTypeSerializer):
-    def define(self):
+    def define(self, **_):
         pass
 
     def schema(self):
         return {"type": "binary"}
 
     def serialize(self, obj):
+        # Load binary from disk
+        if isinstance(obj, str) and os.path.isfile(obj):
+            with open(obj, "rb") as f:
+                obj = f.read()
+
         try:
             output = base64.b64encode(obj).decode("ascii")
         except TypeError:
@@ -195,7 +226,7 @@ class BinaryType(ModelTypeSerializer):
 
 
 class NumpyType(ModelTypeSerializer):
-    def define(self, shape=None, dtype=float):
+    def define(self, shape=None, dtype=float, **_):
         self.shape = shape
         if isinstance(dtype, type):
             self.dtype = dtype
@@ -239,7 +270,7 @@ class NumpyType(ModelTypeSerializer):
 
 
 class DataframeType(ModelTypeSerializer):
-    def define(self, max_rows=None, max_cols=None):
+    def define(self, max_rows=None, max_cols=None, **_):
         self.max_rows = max_rows
         self.max_cols = max_cols
 
@@ -251,9 +282,12 @@ class DataframeType(ModelTypeSerializer):
         }
 
     def serialize(self, obj):
+        if isinstance(obj, list):
+            return obj
+
         return obj.to_dict()
 
-    def deserialize(self, data):
+    def deserialize(self, data):  # noqa
         try:
             pd = import_module("pandas")
         except ModuleNotFoundError:
@@ -264,6 +298,11 @@ class DataframeType(ModelTypeSerializer):
                 obj = pd.read_json(data)
             except ValueError:
                 raise InvalidTypeException("invalid_dataframe_str")
+        elif isinstance(data, list):
+            try:
+                obj = pd.DataFrame(data)
+            except ValueError:
+                raise InvalidTypeException("invalid_dataframe_list")
         elif isinstance(data, dict):
             try:
                 obj = pd.DataFrame.from_dict(data)
@@ -278,7 +317,7 @@ class DataframeType(ModelTypeSerializer):
 
 
 class TensorType(ModelTypeSerializer):
-    def define(self, shape=None, dtype=float):
+    def define(self, shape=None, dtype=float, **_):
         self.shape = shape
         self.dtype = dtype
 
@@ -313,7 +352,52 @@ class TensorType(ModelTypeSerializer):
         return obj
 
 
+class BooleanType(ModelTypeSerializer):
+    """
+    In the case of types where both the serialized and deserialized values are
+    valid in both JSON payloads, as well as python objects, we can re-use the same
+    logic for serialize/deserialize. For example, you can pass a boolean object
+    as JSON, but you can also use that type directly in application logic as a boolean.
+    """
+
+    def define(self, **_):
+        pass
+
+    def schema(self):
+        return {"type": "boolean"}
+
+    def _to_boolean(self, obj):
+        if isinstance(obj, bool):
+            return obj
+
+        elif isinstance(obj, str):
+            if obj.lower() == "true":
+                return True
+            elif obj.lower() == "false":
+                return False
+            else:
+                raise InvalidTypeException("invalid_boolean_value")
+
+        elif isinstance(obj, int) or isinstance(obj, float):
+            obj = int(obj)
+            if obj == 1:
+                return True
+            elif obj == 0:
+                return False
+            else:
+                raise InvalidTypeException("invalid_boolean_value")
+
+        return obj
+
+    def serialize(self, obj):
+        return self._to_boolean(obj)
+
+    def deserialize(self, data):
+        return self._to_boolean(data)
+
+
 class ModelTypes:
+    Boolean = BooleanType
     Float = FloatType
     String = StringType
     Json = JsonType
@@ -332,8 +416,13 @@ class ModelTypes:
             raise InvalidTypeException("invalid_object_format")
 
         for key in schema.keys():
-            if key not in objects.keys():
+            if key not in objects.keys() and getattr(schema[key], "required", True):
                 errors.append(f"{key}:required")
+            elif (
+                key not in objects.keys()
+                and getattr(schema[key], "required", True) is False  # noqa
+            ):
+                continue
             else:
                 _type = schema[key]
                 _input = objects[key]
@@ -357,8 +446,13 @@ class ModelTypes:
             raise InvalidTypeException("invalid_data_format")
 
         for key in schema.keys():
-            if key not in data.keys():
+            if key not in data.keys() and getattr(schema[key], "required", True):
                 errors.append(f"{key}:required")
+            elif (
+                key not in data.keys()
+                and getattr(schema[key], "required", True) is False  # noqa
+            ):
+                continue
             else:
                 _type = schema[key]
                 _input = data[key]
@@ -394,6 +488,7 @@ class ModelTypes:
             "dataframe": DataframeType,
             "image": ImageType,
             "binary": BinaryType,
+            "boolean": BooleanType,
         }
 
         schema = {}
